/**
 * UGENE - Integrated Bioinformatics Tools.
 * Copyright (C) 2008-2021 UniPro <ugene@unipro.ru>
 * http://ugene.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 */

#ifndef _U2_DISABLE_WARNINGS_H_
#define _U2_DISABLE_WARNINGS_H_

/***
 * Once included this file disables all warnings in the current compilation unit.
 * This utility should be used to suppress warnings in 3rd party code included into UGENE.
 *
 * Note: never include this file into files owned/created by UGENE (both headers and cpp) because
 *  it will disable warnings in our code too.
 */

#ifdef __GNUC__
#    pragma GCC diagnostic push
#    pragma GCC diagnostic ignored "-Wunused-parameter"
#    pragma GCC diagnostic ignored "-Wunused-function"
#    pragma GCC diagnostic ignored "-Wmisleading-indentation"
#    pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif

/**
 * This macro does nothing today but helps to avoid "unused include" warning for #include <U2Core/disable-warnings.h>:
 *
 * Usage:
 * #include <U2Core/disable-warnings>
 * U2_DISABLE_WARNINGS
 */
#define U2_DISABLE_WARNINGS

#endif
